@@ -1,4 +1,4 @@
-{-# LANGUAGE CPP, TupleSections, BangPatterns #-}
+{-# LANGUAGE CPP, TupleSections, BangPatterns, ConstraintKinds #-}
 {-# OPTIONS_GHC -fno-warn-duplicate-exports #-}
 
 -- | This module extends "Data.List" with extra functions of a similar nature.
@@ -9,10 +9,11 @@ module Data.List.Extra(
     module Data.List,
     -- * String operations
     lower, upper, trim, trimStart, trimEnd, word1, line1,
-    -- * Splitting    
+    -- * Splitting
     dropEnd, takeEnd, splitAtEnd, breakEnd, spanEnd,
     dropWhileEnd, dropWhileEnd', takeWhileEnd,
     stripSuffix, stripInfix, stripInfixEnd,
+    dropPrefix, dropSuffix,
     wordsBy, linesBy,
     breakOn, breakOnEnd, splitOn, split, chunksOf,
     -- * Basics
@@ -21,6 +22,8 @@ module Data.List.Extra(
     groupSort, groupSortOn, groupSortBy,
     nubOrd, nubOrdBy, nubOrdOn,
     nubOn, groupOn, sortOn,
+    nubSort, nubSortBy, nubSortOn,
+    maximumOn, minimumOn,
     disjoint, allSame, anySame,
     repeatedly, for, firstJust,
     concatUnzip, concatUnzip3,
@@ -28,6 +31,7 @@ module Data.List.Extra(
     replace, merge, mergeBy,
     ) where
 
+import Partial
 import Data.List
 import Data.Maybe
 import Data.Function
@@ -179,13 +183,14 @@ splitAtEnd i xs = f xs (drop i xs)
 --
 -- > \i xs -> zip [i..] xs == zipFrom i xs
 -- > zipFrom False [1..3] == undefined
-zipFrom :: Enum a => a -> [b] -> [(a, b)]
+zipFrom :: (Partial, Enum a) => a -> [b] -> [(a, b)]
 zipFrom = zipWithFrom (,)
 
 -- | 'zipFrom' generalised to any combining operation.
+--   Never truncates the output - raises an error if the enumeration runs out.
 --
 -- > \i xs -> zipWithFrom (,) i xs == zipFrom i xs
-zipWithFrom :: Enum a => (a -> b -> c) -> a -> [b] -> [c]
+zipWithFrom :: (Partial, Enum a) => (a -> b -> c) -> a -> [b] -> [c]
 zipWithFrom f a xs = go a xs
     where
         -- if we aren't strict in the accumulator, it's highly like to be a space leak
@@ -294,6 +299,13 @@ groupOn f = groupBy ((==) `on2` f)
 nubOn :: Eq b => (a -> b) -> [a] -> [a]
 nubOn f = map snd . nubBy ((==) `on` fst) . map (\x -> let y = f x in y `seq` (y, x))
 
+-- | A version of 'maximum' where the comparison is done on some extracted value.
+maximumOn :: Ord b => (a -> b) -> [a] -> a
+maximumOn f = maximumBy (compare `on` f)
+
+-- | A version of 'minimum' where the comparison is done on some extracted value.
+minimumOn :: Ord b => (a -> b) -> [a] -> a
+minimumOn f = minimumBy (compare `on` f)
 
 -- | A combination of 'group' and 'sort'.
 --
@@ -340,7 +352,7 @@ mergeBy f (x:xs) (y:ys)
 -- > replace "el" "e" "Hello"       == "Helo"
 -- > replace "" "e" "Hello"         == undefined
 -- > \xs ys -> not (null xs) ==> replace xs xs ys == ys
-replace :: Eq a => [a] -> [a] -> [a] -> [a]
+replace :: (Partial, Eq a) => [a] -> [a] -> [a] -> [a]
 replace [] _ _ = error "Extra.replace, first argument cannot be empty"
 replace from to xs | Just xs <- stripPrefix from xs = to ++ replace from to xs
 replace from to (x:xs) = x : replace from to xs
@@ -425,7 +437,7 @@ mconcatMap f = mconcat . map f
 -- The first element of the returned tuple
 -- is the prefix of @haystack@ before @needle@ is matched.  The second
 -- is the remainder of @haystack@, starting with the match.
--- If you want the remainder /without/ the patch, use 'stripInfix'.
+-- If you want the remainder /without/ the match, use 'stripInfix'.
 --
 -- > breakOn "::" "a::b::c" == ("a", "::b::c")
 -- > breakOn "/" "foobar"   == ("foobar", "")
@@ -457,7 +469,7 @@ breakOnEnd needle haystack = both reverse $ swap $ breakOn (reverse needle) (rev
 -- > splitOn "x"    ""                 == [""]
 -- > \s x -> s /= "" ==> intercalate s (splitOn s x) == x
 -- > \c x -> splitOn [c] x                           == split (==c) x
-splitOn :: Eq a => [a] -> [a] -> [[a]]
+splitOn :: (Partial, Eq a) => [a] -> [a] -> [[a]]
 splitOn [] _ = error "splitOn, needle may not be empty"
 splitOn _ [] = [[]]
 splitOn needle haystack = a : if null b then [] else splitOn needle $ drop (length needle) b
@@ -501,8 +513,27 @@ dropWhileEnd p = foldr (\x xs -> if p x && null xs then [] else x : xs) []
 dropWhileEnd' :: (a -> Bool) -> [a] -> [a]
 dropWhileEnd' p = foldr (\x xs -> if null xs && p x then [] else x : xs) []
 
--- | Return the prefix of the second string if its suffix
--- matches the entire first string.
+
+-- | Drops the given prefix from a list.
+--   It returns the original sequence if the sequence doesn't start with the given prefix.
+--
+-- > dropPrefix "Mr. " "Mr. Men" == "Men"
+-- > dropPrefix "Mr. " "Dr. Men" == "Dr. Men"
+dropPrefix :: Eq a => [a] -> [a] -> [a]
+dropPrefix a b = fromMaybe b $ stripPrefix a b
+
+
+-- | Drops the given suffix from a list.
+--   It returns the original sequence if the sequence doesn't end with the given suffix.
+--
+-- > dropSuffix "!" "Hello World!"  == "Hello World"
+-- > dropSuffix "!" "Hello World!!" == "Hello World!"
+-- > dropSuffix "!" "Hello World."  == "Hello World."
+dropSuffix :: Eq a => [a] -> [a] -> [a]
+dropSuffix a b = fromMaybe b $ stripSuffix a b
+
+-- | Return the prefix of the second list if its suffix
+--   matches the entire first list.
 --
 -- Examples:
 --
@@ -541,11 +572,34 @@ stripInfixEnd needle haystack = both reverse . swap <$> stripInfix (reverse need
 -- > chunksOf 3 "mytest"  == ["myt","est"]
 -- > chunksOf 8 ""        == []
 -- > chunksOf 0 "test"    == undefined
-chunksOf :: Int -> [a] -> [[a]]
+chunksOf :: Partial => Int -> [a] -> [[a]]
 chunksOf i xs | i <= 0 = error $ "chunksOf, number must be positive, got " ++ show i
 chunksOf i xs = repeatedly (splitAt i) xs
 
 
+-- | /O(n log n)/. The 'nubSort' function sorts and removes duplicate elements from a list.
+-- In particular, it keeps only the first occurrence of each element.
+--
+-- > nubSort "this is a test" == " aehist"
+-- > \xs -> nubSort xs == nub (sort xs)
+nubSort :: Ord a => [a] -> [a]
+nubSort = nubSortBy compare
+
+-- | A version of 'nubSort' which operates on a portion of the value.
+--
+-- > nubSortOn length ["a","test","of","this"] == ["a","of","test"]
+nubSortOn :: Ord b => (a -> b) -> [a] -> [a]
+nubSortOn f = nubSortBy (compare `on` f)
+
+-- | A version of 'nubSort' with a custom predicate.
+--
+-- > nubSortBy (compare `on` length) ["a","test","of","this"] == ["a","of","test"]
+nubSortBy :: (a -> a -> Ordering) -> [a] -> [a]
+nubSortBy cmp = f . sortBy cmp
+    where f (x1:x2:xs) | cmp x1 x2 == EQ = f (x1:xs)
+          f (x:xs) = x : f xs
+          f [] = []
+
 -- | /O(n log n)/. The 'nubOrd' function removes duplicate elements from a list.
 -- In particular, it keeps only the first occurrence of each element.
 -- Unlike the standard 'nub' operator, this version requires an 'Ord' instance
