@@ -1,34 +1,40 @@
 {-# LANGUAGE ScopedTypeVariables, CPP, FlexibleInstances #-}
+{-# OPTIONS_GHC -fno-warn-orphans #-} -- OK because a test module
 
-module TestUtil(runTests, testGen, testRaw, erroneous, erroneousIO, (====), module X) where
+module TestUtil
+    (runTests
+    ,testGen, testRaw
+    ,erroneous, erroneousIO
+    ,(====), (==>)
+    ,ASCIIString(..)
+    ,module X
+    ) where
 
 import Test.QuickCheck
 import Test.QuickCheck.Test
-import Control.Exception.Extra
-import Data.Either.Extra
-import System.IO.Extra
-import Data.Version.Extra
-import Data.IORef
 import System.IO.Unsafe
-import Data.Time.Clock
-import Data.Time.Calendar
 import Text.Show.Functions()
 
-import Extra as X
 import Control.Applicative as X
-import Control.Monad as X
-import Data.Function as X
-import Data.List as X
+import Control.Concurrent.Extra as X
+import Control.Exception.Extra as X
+import Control.Monad.Extra as X
 import Data.Char as X
+import Data.Either.Extra as X
+import Data.Function as X
+import Data.IORef.Extra as X
+import Data.List.Extra as X
 import Data.Monoid as X
-import Data.Tuple as X
-import Data.Typeable as X
-import Data.Version as X
-import System.Directory as X
+import Data.Tuple.Extra as X
+import Data.Typeable.Extra as X
+import Data.Version.Extra as X
+import Numeric.Extra as X
+import System.Directory.Extra as X
 import System.FilePath as X
-import System.Info as X
-import Control.Exception as X
-import Test.QuickCheck as X((==>))
+import System.Info.Extra as X
+import System.IO.Extra as X
+import System.Process.Extra as X
+import System.Time.Extra as X
 
 
 {-# NOINLINE testCount #-}
@@ -70,17 +76,18 @@ runTests t = do
     n <- readIORef testCount
     putStrLn $ "Success (" ++ show n ++ " tests)"
 
-instance Testable () where
-    property = property . (`seq` True)
-
 instance Testable a => Testable (IO a) where
     property = property . unsafePerformIO
 
-instance Eq a => Eq (IO a) where
+-- We only use this property to assert equality as a property
+-- And the Show instance is useless (since it may be non-deterministic)
+-- So we print out full information on failure
+instance (Show a, Eq a) => Eq (IO a) where
     a == b = unsafePerformIO $ do
         a <- try_ $ captureOutput a
         b <- try_ $ captureOutput b
-        return $ a == b
+        if a == b then return True else
+            error $ show ("IO values not equal", a, b)
 
 instance Show (IO a) where
     show _ = "<<IO>>"
@@ -95,12 +102,3 @@ instance Arbitrary a => Arbitrary (IO a) where
 
 instance Eq SomeException where
     a == b = show a == show b
-
-instance Arbitrary UTCTime where
-    arbitrary = liftM2 UTCTime arbitrary arbitrary
-
-instance Arbitrary Day where
-    arbitrary = fmap ModifiedJulianDay arbitrary
-
-instance Arbitrary DiffTime where
-    arbitrary = realToFrac <$> choose (0 :: Double, 86401)
